from __future__ import division
import objects.abstract
import game.objects.opengl
from util import MathTuple, inner_ints, otherrange, DefaultList
import data
import math

class MovableGameObject(objects.abstract.Moving):
	""" Interface for the world to interact with this object."""
	def __init__(self, position=MathTuple((0,0)), velocity=MathTuple((0,0))):
		self.position1 = self.position2 = position
		self.velocity = velocity
		self.falling = True
		self.bounce = 0
		self.delete_me = False
		self.is_heavy = True
	
	@classmethod
	def get_painter(cls):
		try:
			return getattr(game.objects.opengl, cls.__name__)
		except AttributeError:
			return super(MovableGameObject, cls).get_painter()

	def tick(self):
		"""is called every tick before dispatching events or querying the acceleration to notify the object that a new tick has begun."""
		pass

	def get_acceleration_for_tick(self):
		"""return where the object would like to accelerate."""
		return MathTuple((0,0))

	def get_collision_box_boundaries(self):
		""" return collision boundaries of the object as (left, right, up, down) """
		p = self.position2
		cp = self.collisionpoints
		return (cp[0][0] +p[0], cp[1][0] + p[0], cp[2][1] + p[1], cp[0][1] + p[1])

class Block(objects.abstract.Paintable):
	""" The acceleration factor is applied to the acceleration of objects that are on top of the block.
		The friction factor is multiplied with the current velocity of objects on top of the block, multiplied by -1 and added to the total acceleration. This is used to limit the speed and slow objects down when they are no longer accelerated. """

	def __init__(self, position, imagefile, friction_factor, acceleration_factor):
		self.position = position
		self.friction_factor = friction_factor
		self.acceleration_factor = acceleration_factor
		self.imagefile = imagefile
	
	@classmethod
	def get_painter(cls):
		return game.objects.opengl.Block # does not inherit from any previous game object; is faster anyway

	def get_collision_box(self):
		return (self.position, (self.position[0]+1, self.position[1]+1))

class Level(objects.abstract.Paintable):
	def __init__(self, file):
		d = [x.strip("\n\r") for x in data.load(["levels","%s.lvl"%file]).readlines()]
		header = d[:d.index("")]
		self.meta = dict(h.split(": ") for h in header)

		self.stars_normal = []
		self.stars_time = []
		self.stars_points = []

		if self.meta['Layout']=='horizontal':
			lines = d[-1:d.index(""):-1] # in an ascii file, the 0th line is on top. in the game, [:,0] is bottom row
			self.tiles = DefaultList(DefaultList(0),[DefaultList(0, [ord(x)-32 for x in l]) for l in lines])
		elif self.meta['Layout']=='vertical':
			filelines = d[d.index("")+1:-1]
			levelheight = len(filelines[0])
			self.tiles = DefaultList(DefaultList(0))
			for _ in range(levelheight): self.tiles.append(DefaultList(0))
			for fileline in filelines:
				for i, c in enumerate(fileline):
					self.tiles[levelheight-i-1].append(ord(c)-32)
		else:
			raise Exception("unknown format")
	
		for y,line in enumerate(self.tiles):
			for x, tile in enumerate(line):
				if tile == ord("*")-32:
					line[x]=0
					self.stars_normal.append((x,y))
				elif tile == ord("T")-32:
					line[x]=0
					self.stars_time.append((x,y))
				elif tile == ord("P")-32:
					line[x]=0
					self.stars_points.append((x,y))
	
	@classmethod
	def get_painter(cls):
		return game.objects.opengl.Level

	def tick(self):
		pass

	def collide(self, o):
		cp = o.collisionpoints
		pos = o.position1
		vector = o.position2 - o.position1
		x_corr=y_corr=0

		#check collision x
		if vector[0] > 0:
			x = int(math.floor(pos[0] + cp[1][0] + vector[0]))
			for y in range(int(math.floor(pos[1] + cp[1][1])), int(math.ceil(pos[1] + cp[2][1]))):
				if bool(self.tiles[y][x]):
					x_corr = x - (pos[0]+cp[1][0] + vector[0])
					break

		elif vector[0] < 0:
			x = int(math.floor(pos[0] + cp[0][0]+ vector[0]))
			for y in range(int(math.floor(pos[1] + cp[0][1])), int(math.ceil(pos[1] + cp[3][1]))):
				if bool(self.tiles[y][x]):
					x_corr = x+1 - (pos[0]+cp[0][0] + vector[0])
					break

		#check collision y
		if vector[1] > 0:
			y = int(math.floor(pos[1] + cp[3][1] + vector[1]))
			for x in range(int(math.floor(pos[0] + cp[3][0])), int(math.ceil(pos[0] + cp[2][0]))):
				if bool(self.tiles[y][x]):
					y_corr = y-(pos[1]+cp[3][1]+vector[1])
					break
		elif vector[1] < 0:
			y = int(math.floor(pos[1] + cp[0][1] + vector[1]))
			for x in range(int(math.floor(pos[0] + cp[0][0])), int(math.ceil(pos[0] + cp[1][0]))):
				if bool(self.tiles[y][x]):
					y_corr = y+1 - (pos[1]+cp[0][1]+vector[1])
					break


		return (x_corr, y_corr)

	def acc_factor_at(self, coords):
		return 1

	def fric_factor_at(self, coords):
		return 0.1

