class MenuItemList(list):
	def __getitem__(self, item):
		if isinstance(item, str):
			matching = [x for x in self if x.label == item]
			return matching[0]
		else:
			return super(MenuItemList, self).__getitem__(item)

class MenuItem(object):
	def __init__(self, label, choices=None, callback=None):
		"""item in a menu scene. choices is a list of strings to be displayed. callback is called on change."""
		self.label = label
		self.choices = choices
		if choices:
			self.chosen = 0
		self.callback = callback
		self.disabled=False
	
	def get_chosen(self):
		return self.choices[self.chosen]

class MenuItemSeparator(object):
	label="-----"
	disabled=True
	choices=None

from scenes import Scene
from objects.abstract import Text
from util import MathTuple
class MenuScene(Scene):
	inactive_color=(255,255,255,255)
	active_color=(255,0,0,255)

	def __init__(self, title, items, *args, **kwords):
		"""simple menu with menu title, offering choices out of a list of MenuItems"""
		super(MenuScene, self).__init__(*args, **kwords)
		self.items = items

		self.title = Text()
		self.title.text = title

		self.focus = 0
		self.labels = []
		self.values = []

		start = MathTuple((4,46))
		line = MathTuple((0,-3))
		tab = MathTuple((30,0))

		self.title.position = start
		self.title.size = 4

		for n,x in enumerate(self.items):
			label = Text()
			label.text = x.label
			label.position = start+line*(n+2)
			label.size = 3
			self.labels.append(label)

			if x.choices:
				value = Text()
				value.text = x.get_chosen()
				value.position = start+line*(n+2)+tab
				value.size = 3
			else:
				value = None
			self.values.append(value)
				
		self.update_items()

	def step(self, now):
		self.environment.screen.view_area((0,60),(80,0))

		self.handle_events()
		self.update_items()

		self.title.paint()
		for x in self.labels:
			x.paint()
		for x in self.values:
			if x:
				x.paint()
	
	def handle_events(self):
		events = self.environment.events

		for e in events.get():
			if e.type == events.KEYDOWN:
				if e.key == events.K_DOWN:
					self.focus = (self.focus+1) % len(self.items)
					while self.items[self.focus].disabled:
						self.focus = (self.focus+1) % len(self.items)
				elif e.key == events.K_UP:
					self.focus = (self.focus-1) % len(self.items)
					while self.items[self.focus].disabled:
						self.focus = (self.focus-1) % len(self.items)
				elif e.key in (events.K_RIGHT, events.K_RETURN, events.K_SPACE, events.K_PAGEDOWN):
					item = self.items[self.focus]
					if item.choices:
						item.chosen = (item.chosen+1) % len(item.choices)
						if item.callback:
							item.callback(self)
					else:
						if e.key == events.K_RETURN and item.callback:
							item.callback(self)
				elif e.key in (events.K_LEFT, events.K_PAGEUP):
					item = self.items[self.focus]
					if item.choices:
						item.chosen = (item.chosen-1) % len(item.choices)
						if item.callback:
							item.callback(self)

	
	def update_items(self):
		for n,x in enumerate(self.items):
			self.labels[n].color = n==self.focus and self.active_color or self.inactive_color
			if self.values[n]:
				self.values[n].text = x.get_chosen()
				self.values[n].color = n==self.focus and self.active_color or self.inactive_color
