from __future__ import division
import warnings
from OpenGL.GL import *
import pygame
import data
from objects.opengl_auxiliary import *

class Painter(object):
	@staticmethod
	def interpolate(phi, a, b):
		return a+(b-a)*phi
	@classmethod
	def paint(cls, abstract, phi=0):
		"""generic painter method that respects coordinates and transforms accordingly.
		overwrite for unpositioned painters."""
		if hasattr(abstract, "position"): # a _unique_ position
			pos = abstract.position
		elif hasattr(abstract, "position1"):
			pos=cls.interpolate(phi, abstract.position1, abstract.position2)
		else:
			pos=None
		if pos:
			glPushMatrix()
			glTranslate(pos[0], pos[1], 0)
		cls.paint_transformed(abstract, phi)
		if pos:
			glPopMatrix()
	@staticmethod
	def tick(*args, **kwords):
		"""implemente a method to do something every tick (eg varying textures over many ticks)"""

def paint_unknown_symbol():
	glColor(1,0,1)
	rectquad((-0.05,0.05),(0.05,-0.05))
	rectquad((-0.005,1),(0.005,-1))
	rectquad((-1,0.005),(1,-0.005))

class Unknown(Painter):
	@staticmethod
	def paint_transformed(abstract, phi=0):
		paint_unknown_symbol()

class CoordinateSystem(Painter):
	@staticmethod
	def paint(abstract, phi=0):
		glColor(1,1,1)

		rectquad((-1,0.1),(1,-0.1))
		rectquad((-0.1,1),(0.1,-1))
		if abstract.show_terminators:
			rectquad((-1,0.4),(-0.9,-0.4))
			rectquad((0.9,0.4),(1,-0.4))
			rectquad((-0.4,1),(0.4,0.9))
			rectquad((-0.4,-0.9),(0.4,-1))

		glColor(1,0,0)
		glBegin(GL_TRIANGLES)
		glVertex(1,1)
		glVertex(0.9,1)
		glVertex(1,0.9)
		glEnd()

class StaticSprite(Painter):
	"""sprite painter for a single sprite per class"""

	imagefile=None
	size=None

	_texture_id=0

	@classmethod
	def paint_transformed(cls, abstract, phi=0):
		if not cls.imagefile or not cls.size:
			warnings.warn("no configuration for %s, using Unknown")
			Unknown.paint_transformed(abstract, phi)
			return

		if not cls._texture_id:
			cls._texture_id=glGenTextures(1)
			glBindTexture(GL_TEXTURE_2D, cls._texture_id)

			map_to_current_texture(["StaticSprite",cls.imagefile])
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
		else:
			glBindTexture(GL_TEXTURE_2D, cls._texture_id)

		glColor(1,1,1)
		rectquad(cls.size[0],cls.size[1],True)

		glBindTexture(GL_TEXTURE_2D, 0)

class StaticSprite3D(StaticSprite):
	"""sprite painter for a single sprite per class, with pseudo 3d textures"""
	tickcount = 0
	cycletime = 20 # ticks

	@classmethod
	def tick(cls):
		cls.tickcount+=1

	@classmethod
	def _which_image(cls, abstract, phi):
		"""return (mirror_x, mirror_y, which_image)"""
		if callable(cls.cycletime):
			cycletime = cls.cycletime(abstract)
		else:
			cycletime = cls.cycletime

		if hasattr(cls, "subset"):
			subset = cls.subset(abstract)
		else:
			subset = [(False, False, x) for x in xrange(cls.depth)]

		incycle = int((cls.tickcount+phi)%cycletime / cycletime * len(subset))

		return subset[incycle]

	@classmethod
	def paint_transformed(cls, abstract, phi):
		if not cls.imagefile or not cls.size or not cls.depth:
			warnings.warn("no configuration for %s, using Unknown")
			Unknown.paint_transformed(abstract, phi)
			return

		if not cls._texture_id:
			cls._texture_id=glGenTextures(cls.depth)
			file = data.load(["StaticSprite",cls.imagefile])
			image = pygame.image.load(file)
			partwidth = image.get_size()[0]//cls.depth
			height = image.get_size()[1]
			for z in range(cls.depth):
				part = image.subsurface((z*partwidth,0,partwidth,height))

				glBindTexture(GL_TEXTURE_2D, cls._texture_id[z])

				strdata = pygame.image.tostring(part, "RGBA")
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, partwidth, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, strdata)

				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)

		mirror_x, mirror_y, which_image = cls._which_image(abstract, phi)
		glBindTexture(GL_TEXTURE_2D, cls._texture_id[which_image])

		glColor(1,1,1)
		if mirror_y:
			if not mirror_x:
				rectquad(cls.size[0],cls.size[1],True,(0,1,1,0))
			else:
				rectquad(cls.size[0],cls.size[1],True,(0,0,1,1))
		else:
			if not mirror_x:
				rectquad(cls.size[0],cls.size[1],True)
			else:
				rectquad(cls.size[0],cls.size[1],True,(1,1,0,0))

		glBindTexture(GL_TEXTURE_2D, 0)

class Lenna(StaticSprite):
	size=((-0.5,0.5),(0.5,-0.5))
	imagefile="lenna.png"

class _Font(object):
	__slots__ = ['texture', 'index', 'aspect']
class Text(Painter):
	known_fonts={}

	@classmethod
	def paint_transformed(cls, abstract, phi=0):
		if abstract.font not in cls.known_fonts:
			f = _Font()
			f.texture = glGenTextures(1)
			glBindTexture(GL_TEXTURE_2D, f.texture)
			cls.known_fonts[abstract.font] = f

			indexdata = data.load(['fonts','%s.txt'%abstract.font]).read().strip()
			f.index = dict(zip(indexdata, xrange(len(indexdata))))

			(width, height), cls._correction_coords = map_to_pow2_current_texture(['fonts','%s.png'%abstract.font])
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)

			f.aspect = width / (height * len(f.index) )
		else:
			f = cls.known_fonts[abstract.font]
			glBindTexture(GL_TEXTURE_2D, f.texture)

		glColor(*abstract.color)

		glPushMatrix()
		
		# potential speedup: cache (font, text) in display list
		count = len(f.index)
			
		cheight = abstract.size
		cwidth = cheight * f.aspect

		if abstract.align=="right":
			glTranslate(-cwidth*len(abstract.text), 0, 0)
		elif abstract.align=="center":
			glTranslate(-cwidth*len(abstract.text)/2, 0, 0)

		corrx,corry = cls._correction_coords
		for l in abstract.text:
			if l==" ":
				glTranslate(cwidth, 0, 0)
				continue


			offset=f.index[l]

			rectquad(
					(0,cheight/2),(cwidth,-cheight/2),
					True,
					(1*corry,(offset)/count*corrx,0,(offset+1)/count*corrx)
					)
			glTranslate(cwidth, 0, 0)
		glPopMatrix()

		glBindTexture(GL_TEXTURE_2D, 0)
