import pygame
import pygame.locals
import pygame.mouse
from util import *
import math

class Events(object):
	""" Responsible for handling application input. This includes keyboard and mouse as well as windowing events."""

	from pygame.locals import *
	def __init__(self):
		#set up pygame event filtering, we only care about the QUIT event, keyboard input and mouse
		#button presses. The rest is handled via special input device methods
		pygame.event.set_allowed([
			pygame.locals.KEYUP,
			pygame.locals.KEYDOWN,
			pygame.locals.QUIT,
			pygame.locals.MOUSEBUTTONUP,
			pygame.locals.MOUSEBUTTONDOWN,
			pygame.locals.MOUSEMOTION
			])

	def get(self):
		return pygame.event.get()

	def get_mouse_pos(self):
		return pygame.mouse.get_pos()

	
class InputSchema(object):
	""" This class handles distribution of events to triggers. A scene that needs advanced input should have one InputSchema active at a time and just needs to register trigger via register and than call process_input with the pygame events at the beginning of the tick and clear_after_tick after the end of the tick. """
	def __init__(self):
		self.all_trigger = []
		self.event_map = {}

	def register(self, trigger):
	
		for key in trigger.keys:
			if self.event_map.has_key(key):
				self.event_map[key].append(trigger)
			else:
				self.event_map[key] = [trigger]
		self.all_trigger.append(trigger)

	def process_input(self, eventlist):
		for e in eventlist:
			if e.type == pygame.locals.MOUSEBUTTONUP:
				params = { 'key': e.button, 'down':False, 'pos': e.pos }
			elif e.type == pygame.locals.MOUSEBUTTONDOWN:
				params = { 'key': e.button, 'down':True, 'pos': e.pos }
			elif e.type == pygame.locals.KEYDOWN:
				params = { 'key': e.key, 'down': True, 'unicode': e.unicode }
			elif e.type == pygame.locals.KEYUP:
				params = { 'key': e.key, 'down': False }
			elif e.type == pygame.locals.QUIT:
				params = { 'key': e.type }
			elif e.type == pygame.locals.MOUSEMOTION:
				params = { 'key': e.type, 'pos': e.pos }
			else:
				continue

			if self.event_map.has_key(params['key']):
				for trigger in self.event_map[params['key']]:
					trigger.handle_key_event(**params)

	def clear_after_tick(self):
		for trigger in self.all_trigger:
			trigger.clear_after_tick()

	def reset(self):
		for trigger in self.all_trigger:
			trigger.reset()




class EventTrigger(object):
	def __init__(self, keys):
		if hasattr(keys, '__iter__'):
			self.keys = keys
		else:
			self.keys = [keys]

	def reset(self):
		""" the event system might have lost some events so reset everything to a sane state"""

	def clear_after_tick(self):
		""" clear the state of this tick. this is called before the tick ends so nothing spills to the next tick """

	def occured(self):
		return False


class KbdTap(EventTrigger):
	""" Triggers once when a key is pressed and ignores further KEYDOWN events until a KEYUP is received. """
	def __init__(self, *args, **kwords):
		super(KbdTap, self).__init__(*args, **kwords)
		self.reset()

	def reset(self):
		self.is_pressed = False
		self.has_occured = False

	def handle_key_event(self, key, down, **kword):
		if not self.is_pressed and down: 
			self.has_occured = True 
			self.is_pressed = True

		if not down: 
			self.is_pressed = False

	def clear_after_tick(self):
		self.has_occured = False

	def occured(self):
		return self.has_occured

class KbdIsPressed(EventTrigger):
	""" Occurs as long as a key is down. Use for walking etc. """
	def __init__(self, *args, **kwords):
		super(KbdIsPressed, self).__init__(*args, **kwords)
		self.reset()

	def reset(self):
		self.is_pressed = False

	def handle_key_event(self, key, down, **kword):
		self.is_pressed = down

	def occured(self):
		return self.is_pressed

class KbdSimultaneousTap(EventTrigger):
	""" Occurs when a list of keys is tapped together in one tick. """
	def __init__(self, *args, **kwords):
		super(KbdSimultaneousTap, self).__init__(*args, **kwords)
		self.sub_trigger = []
		for key in self.keys: 
			self.sub_trigger.append(KbdTap(key))

		self.reset()

	def reset(self):
		for trigger in self.sub_trigger: 
			trigger.reset()
		
	def clear_after_tick(self):
		for trigger in self.sub_trigger: 
			trigger.clear_after_tick()

	def handle_key_event(self, key, down, **kword):
		self.sub_trigger[self.keys.index(key)].handle_key_event(key, down)

	def occured(self):
		for trigger in self.sub_trigger:
			if not trigger.occured():
				return False
		return True

class MouseDragTrigger(EventTrigger):
	""" Triggers when the mouse is moved and tries to maintain an absolute mouse position with respect to the screen """
	def __init__(self, *args, **kwords):
		super(MouseDragTrigger, self).__init__(*args, **kwords)
		self.reset()

	def reset(self):
		self.start = self.end = MathTuple(0,0)
		self.is_dragging = False
		self.has_occured = False

	def handle_key_event(self, pos, down, **kword):
		if not self.is_dragging and down:
			self.start = MathTuple(pos)
			self.is_dragging = True

		elif self.is_dragging and not down:
			self.end = MathTuple(pos)
			dist = self.start - self.end
			self.is_dragging = False
			if abs(dist[0] + dist[1]) > 6:
				self.has_occured = True
			

	def clear_after_tick(self):
		self.has_occured = False

	def occured(self):
		return self.has_occured

class MouseTap(EventTrigger):
	""" Triggers when mouse is clicked"""
	def __init__(self, *args, **kwords):
		super(MouseTap, self).__init__(*args, **kwords)
		self.reset()

	def reset(self):
		self.is_down = False
		self.has_occured = False

	def handle_key_event(self, pos, down, **kword):
		if not self.is_down and down:
			self.is_down  = True
			self.has_occured = True
		
		if self.is_down and not down:
			self.is_down = False
	
	def occurred(self):
		return self.has_occured

	def clear_after_tick(self):
		self.has_occured = False
