from __future__ import division

import objects.abstract

class Scene(object):
	"""scene object

	finally inherited to subclasses like "intro", "menu" or "game".
	completely independent as far as internal logics are concerned, controlled from an environment (or a parent scene playing environment) using the callback step() and event based callbacks.
	"""

	def __init__(self, environment):
		self.environment = environment

	def step(self, now):
		"""implement a function that is called as often as possible, with a milliseconds time passed"""

class TickScene(Scene):
	"""scene which needs ticks at intervals

	on step(), tick() is called at specified intervals, and paint(phi) is called in between.
	"""

	tick_interval = 100

	def __init__(self, *args, **kwords):
		super(TickScene, self).__init__(*args, **kwords)
		self.last_tick = None

	def step(self, now):
		if not self.last_tick:
			self.last_tick=now-self.tick_interval # ensure tick() is always called first
		while now-self.tick_interval>=self.last_tick: # will act like an if most of the time
			self.tick()
			self.last_tick+=self.tick_interval
		self.paint((now-self.last_tick)/self.tick_interval)

	def paint(self, phi):
		"""implement a function that paints the current scene at the position between two ticks at phi of the time between the ticks.
		whether the first tick is the current and the second tick is the anticipated next or the first tick is the last and the second is the current is left to the scene."""
	
	def tick(self):
		"""implement a function that does whatever has to be done every tick"""

from util import *
from pygame_io.events import *
import random
import pygame.locals

class DemoScene(TickScene):
	tick_interval = 100

	def __init__(self, *args, **kwords):
		super(DemoScene, self).__init__(*args, **kwords)

		self.coordinate_system=objects.abstract.CoordinateSystem(True)
		self.fnord=objects.abstract.Fnord()

		self.lenna=objects.abstract.Lenna()
		self.lenna.position=MathTuple((-1,1))

		self.text=objects.abstract.Text()
		self.text.text="press t to change text"
		# make a positioned thing movable (hack; better way?)
		del self.text.position
		self.text.position2 = self.text.position1 = MathTuple((0,1))
		self.text.size=0.3
		self.text.align = "center"

		self.input_schema = InputSchema()

		self.left = KbdTap()
		self.right = KbdTap()
		self.up = KbdTap()
		self.down = KbdTap()
		self.music = KbdTap()
		self.change_text = KbdIsPressed()
		self.exit = KbdSimultaneousTap([pygame.locals.K_u, pygame.locals.K_i])
		self.menu = KbdTap()

		self.input_schema.register(self.left, [pygame.locals.K_a])
		self.input_schema.register(self.right, [pygame.locals.K_d])
		self.input_schema.register(self.up, [pygame.locals.K_w])
		self.input_schema.register(self.down, [pygame.locals.K_s])
		self.input_schema.register(self.music, [pygame.locals.K_SPACE])
		self.input_schema.register(self.change_text, [pygame.locals.K_t])
		self.input_schema.register(self.exit, [pygame.locals.K_u, pygame.locals.K_i])
		self.input_schema.register(self.menu, [pygame.locals.K_RETURN])
		


	def tick(self):
		self.fnord.position1=self.fnord.position2
		self.fnord.position2=MathTuple((random.random()*4-2,random.random()*4-2))

		self.text.position1 = self.text.position2

		self.input_schema.process_input(self.environment.events.get())

		if self.exit.occured():
			raise SystemExit

		if self.left.occured():
			self.lenna.position=MathTuple((-1,0))
		
		if self.down.occured():
			self.lenna.position=MathTuple((0,-1))

		if self.right.occured():
			self.lenna.position=MathTuple((1,0))

		if self.up.occured():
			self.lenna.position=MathTuple((0,1))

		if self.music.occured():
			import data
			self.environment.music.load(data.filepath(("music","demo.mp3")))
			self.environment.music.play()

		#self.environment.screen.zoom=2
		#self.environment.screen.center=(-12.5,-18.75/2) # kinda broken

		if self.change_text.occured():
			self.text.position2 = MathTuple((random.random()*3-2,random.random()*4-2))
			self.text.text=random.choice(["Hello OpenGL",":-)","ESC = exit","WASD moves lenna","space plays sound","z zooms out","m moves screen","return activates menu"])
				

		if self.menu.occured():
			from modules.menu import MenuScene, MenuItem, MenuItemList
			def dump(scene):
				print "you chose %s"%scene.items["Weapon:"].get_chosen()
			def revert(scene, revert_to=self):
				scene.environment.scene=revert_to
					
			items = MenuItemList([MenuItem("noop"), MenuItem("Weapon:",["D6","D10","D12","D20"]), MenuItem("dump config",callback=dump), MenuItem("back",callback=revert)])
			self.environment.scene = MenuScene("main menu", items, self.environment)
			#if e.type == events.MOUSEBUTTONDOWN:
			#	coords = self.environment.screen.translate_coords(e.pos)
			#	print coords

		self.input_schema.clear_after_tick()


	def paint(self, phi):
		screen = self.environment.screen
		screen.view_area((-2*screen.aspect,2),(2*screen.aspect,-2))

		self.coordinate_system.paint(phi)
		self.fnord.paint(phi)
		self.lenna.paint(phi)
		self.text.paint(phi)
